@@ -20,9 +20,9 @@ use pep440_rs::Version;
 use pep508_rs::MarkerEnvironment;
 use platform_tags::Platform;
 use pypi_types::{Metadata23, SimpleJson};
-use uv_auth::KeyringProvider;
 use uv_cache::{Cache, CacheBucket, WheelCache};
 use uv_configuration::IndexStrategy;
+use uv_configuration::KeyringProviderType;
 use uv_normalize::PackageName;
 
 use crate::base_client::{BaseClient, BaseClientBuilder};
@@ -37,7 +37,7 @@ use crate::{CachedClient, CachedClientError, Error, ErrorKind};
 pub struct RegistryClientBuilder<'a> {
     index_urls: IndexUrls,
     index_strategy: IndexStrategy,
-    keyring_provider: KeyringProvider,
+    keyring: KeyringProviderType,
     native_tls: bool,
     retries: u32,
     connectivity: Connectivity,
@@ -52,7 +52,7 @@ impl RegistryClientBuilder<'_> {
         Self {
             index_urls: IndexUrls::default(),
             index_strategy: IndexStrategy::default(),
-            keyring_provider: KeyringProvider::default(),
+            keyring: KeyringProviderType::default(),
             native_tls: false,
             cache,
             connectivity: Connectivity::Online,
@@ -78,8 +78,8 @@ impl<'a> RegistryClientBuilder<'a> {
     }
 
     #[must_use]
-    pub fn keyring_provider(mut self, keyring_provider: KeyringProvider) -> Self {
-        self.keyring_provider = keyring_provider;
+    pub fn keyring(mut self, keyring_type: KeyringProviderType) -> Self {
+        self.keyring = keyring_type;
         self
     }
 
@@ -145,7 +145,7 @@ impl<'a> RegistryClientBuilder<'a> {
             .retries(self.retries)
             .connectivity(self.connectivity)
             .native_tls(self.native_tls)
-            .keyring_provider(self.keyring_provider)
+            .keyring(self.keyring)
             .build();
 
         let timeout = client.timeout();
@@ -285,6 +285,7 @@ impl RegistryClient {
             Path::new(&match index {
                 IndexUrl::Pypi(_) => "pypi".to_string(),
                 IndexUrl::Url(url) => cache_key::digest(&cache_key::CanonicalUrl::new(url)),
+                IndexUrl::Path(url) => cache_key::digest(&cache_key::CanonicalUrl::new(url)),
             }),
             format!("{package_name}.rkyv"),
         );
